﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;


namespace opencvDeps
{
    [StructLayout(LayoutKind.Sequential)]
    public struct CvRect
    {
        public int x;
        public int y;
        public int width;
        public int height;
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct CvPoint
    {
        public int x;
        public int y;
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct CvSize
    {
        public int width;
        public int height;
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct CvScalar
    {
        public double val1;
        public double val2;
        public double val3;
        public double val4;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct IplImage
    {
        int nSize;             /* sizeof(IplImage) */
        int ID;                /* version (=0)*/
        int nChannels;         /* Most of OpenCV functions support 1,2,3 or 4 channels */
        int alphaChannel;      /* Ignored by OpenCV */
        int depth;             /* Pixel depth in bits: IPL_DEPTH_8U, IPL_DEPTH_8S, IPL_DEPTH_16S,
                                    IPL_DEPTH_32S, IPL_DEPTH_32F and IPL_DEPTH_64F are supported.  */
        sbyte colorModel1;     /* Ignored by OpenCV */
        sbyte colorModel2;
        sbyte colorModel3;
        sbyte colorModel4;
        sbyte channelSeq1;     /* ditto */
        sbyte channelSeq2;
        sbyte channelSeq3;
        sbyte channelSeq4;
        public int dataOrder;         /* 0 - interleaved color channels, 1 - separate color channels.
                                    cvCreateImage can only create interleaved images */
        public int origin;            /* 0 - top-left origin,
                               1 - bottom-left origin (Windows bitmaps style).  */
        int align;             /* Alignment of image rows (4 or 8).
                               OpenCV ignores it and uses widthStep instead.    */
        public int width;             /* Image width in pixels.                           */
        public int height;            /* Image height in pixels.                          */
        IntPtr roi;    /* Image ROI. If NULL, the whole image is selected. */
        IntPtr maskROI;      /* Must be NULL. */
        IntPtr imageId;                 /* "           " */
        IntPtr tileInfo;  /* "           " */
        int imageSize;         /* Image data size in bytes
                               (==image->height*image->widthStep
                               in case of interleaved data)*/
        public IntPtr imageData;        /* Pointer to aligned image data.         */
        int widthStep;         /* Size of aligned image row in bytes.    */
        int BorderMode1;     /* Ignored by OpenCV.                     */
        int BorderMode2;
        int BorderMode3;
        int BorderMode4;
        int BorderConst1;    /* Ditto.                                 */
        int BorderConst2;
        int BorderConst3;
        int BorderConst4;
        public IntPtr imageDataOrigin;  /* Pointer to very origin of image data
                               (not necessarily aligned) -
                               needed for correct deallocation */
    };            

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct CvFont
    {
        public IntPtr nameFont;   //Qt:nameFont
        public CvScalar color;       //Qt:ColorFont -> cvScalar(blue_component, green_component, red\_component[, alpha_component])
        public int font_face;    //Qt: bool italic         /* =CV_FONT_* */
        public IntPtr  ascii;      /* font data and metrics */
        public IntPtr greek;
        public IntPtr cyrillic;
        public float hscale, vscale;
        public float shear;      /* slope coefficient: 0 - normal, >0 - italic */
        public int thickness;    //Qt: weight               /* letters thickness */
        public float dx;       /* horizontal interval between letters */
        public int         line_type;    //Qt: PointSize
    };
    
    public static partial class OCV
    {
        const string opencvCoreName = "opencv_world340.dll";
        const string opencvHighGUIName = "opencv_world340.dll";
        const string opencvImgProcName = "opencv_world340.dll";


        /*opencv's core*/        
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern CvSize cvGetSize( IntPtr arr );
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void  cvSetZero( IntPtr arr );

        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvInitFont(ref CvFont font, int font_face, double hscale, double vscale, double shear = 0,
            int thickness = 1, int line_type = 8);
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvPutText(IntPtr img, string text, CvPoint org, ref CvFont font, CvScalar color);
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvGetTextSize(IntPtr text_string, ref CvFont font, CvSize[] text_size, int[] baseline);

        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void  cvRectangle( IntPtr img, CvPoint pt1, CvPoint pt2, CvScalar color, int thickness = 1,
            int line_type = 8, int shift = 0);
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void  cvLine( IntPtr img, CvPoint pt1, CvPoint pt2, CvScalar color, int thickness = 1,
            int line_type = 8, int shift = 0 );
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void  cvCircle( IntPtr img, CvPoint center, int radius, CvScalar color, int thickness = 1,
            int line_type = 8, int shift = 0);

        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr cvCreateImage(CvSize size, int depth, int channels);
        [DllImport(opencvCoreName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvReleaseImage (ref IntPtr image );
        
    
        /*opencv's highgui*/
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr cvQueryFrame(IntPtr capture);
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvShowImage (string  name, IntPtr image);
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern int cvWaitKey (int delay = 0);
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvDestroyWindow (string name);

        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr cvCreateCameraCapture( int index );
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr cvCreateFileCapture (string fileName);
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvReleaseCapture( ref IntPtr capture );

        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern IntPtr cvCreateVideoWriter( string filename, int fourcc, double fps, CvSize frame_size, int is_color = 1);
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern int cvWriteFrame ( IntPtr writer, IntPtr image );
        [DllImport(opencvHighGUIName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvReleaseVideoWriter( ref IntPtr writer );

        /*opencv's imgproc*/
        [DllImport(opencvImgProcName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvCvtColor( IntPtr src, IntPtr dst, int code );
        [DllImport(opencvImgProcName, CallingConvention = CallingConvention.Cdecl)]
        public static extern void cvResize( IntPtr src, IntPtr dst, int interpolation = 1);



        /*function synonimys*/
        public static IntPtr cvCaptureFromCAM(int index)
        {
            return cvCreateCameraCapture(index);
        }

        public static IntPtr cvCaptureFromFile(String fileName)
        {
            return cvCreateFileCapture(fileName);
        }

        public static void cvZero(IntPtr arr)
        {
            cvSetZero(arr);
        }

        public static CvScalar CV_RGB(double r, double g, double b)
        {
            return cvScalar(b, g, r, 0);
        }

        public static int CV_FOURCC(sbyte c1, sbyte c2, sbyte c3, sbyte c4)
        {
            return (((c1) & 255) + (((c2) & 255) << 8) + (((c3) & 255) << 16) + (((c4) & 255) << 24));
        }

        public static CvSize cvSize(int width, int height)
        {
            CvSize s;

            s.width = width;
            s.height = height;

            return s;
        }

        public static CvRect cvRect(int x, int y, int width, int height)
        {
            CvRect r;

            r.x = x;
            r.y = y;
            r.width = width;
            r.height = height;

            return r;
        }

        public static CvPoint cvPoint(int x, int y)
        {
            CvPoint p;

            p.x = x;
            p.y = y;

            return p;
        }
        
        public static CvScalar  cvScalar( double val0, double val1 = 0, double val2 = 0, double val3 = 0)
        {
            CvScalar scalar;
            scalar.val1 = val0; scalar.val2 = val1;
            scalar.val3 = val2; scalar.val4 = val3;
            return scalar;    
        }
    }    
}
