﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;
using System.Xml;

namespace platereader
{
    public partial class Form1 : Form
    {

        [StructLayout(LayoutKind.Sequential, Pack = 0)]
        public struct ANPR_OPTIONS
        {
            public byte sign1, sign2, sign3;
            public int min_plate_size; // Минимальная площадь номера
            public int max_plate_size; // Максимальная площадь номера
            public int Detect_Mode; // Режимы детектирования	
            public int max_text_size; // Максимальное количество символов номера + 1
            public int type_number; // Тип автомобильного номера	
            public int flags; // Дополнительные опции
            public IntPtr custom; // Заполняется только для типа ANPR_CUSTOM_TYPE, иначе пусто

            public IntPtr vers; // Используемая версия iANPR SDK. Если не задано (vers = 0), то считается 1.5.

            // Для определения углов поворота вдоль осей воспользуйтесь утилитой persptrans, которую можно найти в составе iANPR SDK.
            // 90.0 означает отсутствие поворота
            public double alpha;		// Поворот вокруг оси X
            public double beta;			// Поворот вокруг оси Y
            public double gamma;		// Поворот вокруг оси Z

            public int max_threads;		// Количество потоков

        };

        public static int[] anprFullTypes = {4, 7, 9,
                                            310, 311,
                                            911};

#if WIN32
        [DllImport("iANPRinterface_vc14_x86.dll", CallingConvention = CallingConvention.StdCall)]
        unsafe public static extern int anprPlateMemoryXML(byte[] in_buffer, int size_buffer, ANPR_OPTIONS Options, 
            StringBuilder xml_buffer, int[] size_xml_buffer);
        [DllImport("iANPR_vc14_x86.dll", CallingConvention = CallingConvention.StdCall)]
        unsafe public static extern void LicenseValue(sbyte[] key);
#elif WIN64
        [DllImport("iANPRinterface_vc14_x64.dll", CallingConvention = CallingConvention.StdCall)]
        unsafe public static extern int anprPlateMemoryXML(byte[] in_buffer, int size_buffer, ANPR_OPTIONS Options, 
            StringBuilder xml_buffer, int[] size_xml_buffer);
        [DllImport("iANPR_vc14_x64.dll", CallingConvention = CallingConvention.StdCall)]
        unsafe public static extern void LicenseValue(sbyte[] key);
#endif

        public Form1()
        {
            InitializeComponent();

            string fileName = "lic.key";
            if (File.Exists(fileName))
            {
                sbyte[] key = new sbyte[8001];
                using (BinaryReader reader = new BinaryReader(File.Open(fileName, FileMode.Open)))
                {
                    for (int i = 0; i < reader.BaseStream.Length; i++)
                        key[i] = reader.ReadSByte();
                }
                LicenseValue(key);
                //textBox1.Text = "Key loaded ok";
            }
            else
                textBox1.Text = "WARNING! File lic.key not found. This may crash program if you use license version of iANPR SDK dlls";
        }


        private void button1_Click(object sender, EventArgs e)
        {
            Stream myStream = null;
            // Выбор файла
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.InitialDirectory = "c:\\";
            openFileDialog1.Filter = "Jpeg files (*.jpg)|*.jpg|Bmp files (*.bmp)|*.bmp|Png files (*.png)|*.png|All files (*.*)|*.*";
            openFileDialog1.FilterIndex = 4;
            openFileDialog1.RestoreDirectory = false;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // Чтение
                try
                {
                    if ((myStream = openFileDialog1.OpenFile()) != null)
                    {
                        using (myStream)
                        {                            
                            int size = (int)myStream.Length;
                            byte[] buffer = new byte[size];

                            using (MemoryStream ms = new MemoryStream())
                            {
                                int read;
                                while ((read = myStream.Read(buffer, 0, buffer.Length)) > 0)
                                {
                                    ms.Write(buffer, 0, read);
                                }                                
                            }
                            // Передача буфера для распознавания (настройки и вызов функции)
                            ANPR_OPTIONS anpr = new ANPR_OPTIONS ();		
		                    anpr.Detect_Mode = 14;
		                    anpr.min_plate_size = 500;
		                    anpr.max_plate_size = 25000;
		                    anpr.max_text_size = 20;		
		                    anpr.type_number = 0;				
		                    anpr.flags = 1;
                            anpr.custom = IntPtr.Zero;
                            anpr.sign1 = (byte)'i'; anpr.sign2 = (byte)'a'; anpr.sign3 = (byte)'1';

                            anpr.vers = Marshal.AllocHGlobal(4);
                            Marshal.WriteByte (anpr.vers, (byte) '1');
                            Marshal.WriteByte (anpr.vers + 1, (byte) '.');
                            Marshal.WriteByte (anpr.vers + 2, (byte) '6');
                            Marshal.WriteByte(anpr.vers + 3, 0);

                            anpr.alpha = 90;
                            anpr.beta = 90;
                            anpr.gamma = 90;

                            anpr.max_threads = 1;                            

                            StringBuilder buffer_builder = new StringBuilder(10000);
                            int[] size_builder = new int[1];
                            size_builder[0] = 10000;
                            int result = anprPlateMemoryXML(buffer, size, anpr, buffer_builder, size_builder);

                            Marshal.FreeHGlobal(anpr.vers);

                            // Чтение XML
                            StringBuilder output = new StringBuilder();
                            using (XmlReader reader = XmlReader.Create(new StringReader(buffer_builder.ToString())))                            
                            {
                                reader.ReadToFollowing("allnumbers");
                                reader.MoveToFirstAttribute();
                                string numbers = reader.Value;
                                output.AppendLine("Количество найденных номеров: " + numbers);
                                int all = Convert.ToInt32(numbers);
                                for (int i = 0; i < all; i++)
                                {
                                    reader.ReadToFollowing("number");
                                    reader.MoveToFirstAttribute();
                                    string num = reader.Value;
                                    reader.MoveToNextAttribute();
                                    string num_coord = reader.Value;
                                    output.AppendLine("Номер: " + num + " (Координаты: " + num_coord + ")");
                                }
                            }
                            // Вывод буфера на экран
                            textBox1.Text = output.ToString();
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
        }

    }
}
